#include "Wire.h"                       // HW supported I2C library
#define MAX6958_addr 0x39               // I2C address 

int16_t MAX6958Setup(void) {
  uint8_t data[] = {    
    1,              // Register address
    0x0F,              // Decode mode for all digits
    0x10,              // Moderate brightness (0-0x3F)
    3,              // Scan limit: 4 digits
    0x21     };           // Clear and enable display
  Wire.beginTransmission(MAX6958_addr); // Start I2C transaction
  Wire.write(data,5);                   // Send initialization data  
  Wire.endTransmission();               // Send data and stop writing
}

int16_t MAX6958WriteReg(uint8_t addr, uint8_t data) {
  Wire.beginTransmission(MAX6958_addr); // Start I2C transaction
  Wire.write(addr);                     // Send register address
  Wire.write(data);                     // send data for register  
  Wire.endTransmission();               // Send data and stop writing
}

int16_t MAX6958WriteInt(uint16_t data) {
  uint8_t d[5];
  for(int i=0; i<4; i++) {
    d[4-i] = data%10;
    data = data/10;
  }  
  d[0] = 0x20;                          // Address of digit 0  
  Wire.beginTransmission(MAX6958_addr); // Start I2C transaction
  Wire.write(d,5);                      // send address and 4 digits  
  Wire.endTransmission();               // Send data and stop writing
}

void setup() {
  Wire.begin();                         // I2C initialisation
  MAX6958Setup();                       // MAX6958 initialisation
  MAX6958WriteReg(7,1);                 // Testmode ON
  delay(2000);                          // Wait a little...
  MAX6958WriteReg(7,0);                 // Testmode OFF  
}

void loop() {
  delay(2000);
  //--- Counting from 0000 to 9999 ---
  for(uint16_t n = 0; n<10000; n++) {
    MAX6958WriteInt(n);
  } 
  //--- Blink segments ---------------  
  MAX6958WriteReg(4,0x21);              // Clear display
  MAX6958WriteReg(1,0);                 // Set "no decode" mode
  for(int i=0; i< 10; i++) {
    MAX6958WriteReg(0x24,1);            // Seg0 on     
    delay(100);   
    MAX6958WriteReg(0x24,0);            // Seg0 off   
    delay(200);
    MAX6958WriteReg(0x24,2);            // Seg1 on     
    delay(100);   
    MAX6958WriteReg(0x24,0);            // Seg1 off   
    delay(200);  
  }

  //--- Blink decimal points ---------
  for(int i=0; i< 10; i++) {
    for(uint8_t j=0x80; j>8; j=j>>1) {
      MAX6958WriteReg(0x24,j);            // one of Seg4..7 on   
      delay(100);   
      MAX6958WriteReg(0x24,0);            // Seg0..7 off   
    }
    for(uint8_t j=0x10; j>0; j=j<<1) {
      MAX6958WriteReg(0x24,j);            // one of Seg4..7 on     
      delay(100);   
      MAX6958WriteReg(0x24,0);            // Seg0..7 off   
    } 
  }
  //--- Back to numerical mode -------
  MAX6958WriteReg(1,0x0F);              // Set "decode" mode
}



